'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ('value' in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _callAction = require('./callAction');

var _callAction2 = _interopRequireDefault(_callAction);

var _types = require('./types');

var _SimpleGraphController = require('./SimpleGraphController');

var _SimpleGraphController2 = _interopRequireDefault(_SimpleGraphController);

var notFoundValue = {};

var Flambeau = (function () {
  /**
   * Creates a new Flambeau store
   * @param  {graph controller} graph The object graph that reducers are attached to.
   */

  function Flambeau() {
    var graph = arguments.length <= 0 || arguments[0] === undefined ? new _SimpleGraphController2['default']() : arguments[0];

    _classCallCheck(this, Flambeau);

    this.resources = {};
    this.graph = graph;
    this.actionSetIDsToActionFunctions = {};
    this.listeners = [];
  }

  /**
   * Attach a set of reducer functions.
   *
   * @param  {String} id      The unique identifier for this particular reducer
   * @param  {Object} reducer The set of reducer functions, most likely made using `import * as ...`
   * @param  {Object} props   The unique attributes to customize the reducer
   */

  _createClass(Flambeau, [{
    key: 'attachReducer',
    value: function attachReducer(id, reducer, props) {
      this.resources[id] = {
        reducer: reducer,
        props: props
      };

      this.graph = this.graph.setUp(id, {
        state: reducer.getInitialState(props)
      });
    }

    /**
     * Attach multiple reducers, with ids as keys, and reducer function sets as values.
     *
     * @param  {Object} idToReducers An object mapping identifiers to reducer function sets
     * @param  {Object} idToProps    (Optional) the unique attributes for each particular reducer
     */
  }, {
    key: 'attachReducers',
    value: function attachReducers(idToReducers) {
      var _this = this;

      var idToProps = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      Object.keys(idToReducers).forEach(function (id) {
        _this.attachReducer(id, idToReducers[id], idToProps[id]);
      });
    }
  }, {
    key: 'dispatch',
    value: function dispatch(_ref) {
      var _this2 = this;

      var actionSetID = _ref.actionSetID;
      var actionID = _ref.actionID;
      var payload = _ref.payload;

      var idsChanged = {};

      Object.keys(this.resources).forEach(function (resourceID) {
        var resource = _this2.resources[resourceID];

        var result = (0, _callAction2['default'])({
          responder: resource.reducer,
          type: _types.ACTION_TYPE,
          initialValue: _this2.graph.get(resourceID),
          props: resource.props,
          payload: payload,
          notFoundValue: notFoundValue,
          actionSetID: actionSetID,
          actionID: actionID
        });

        if (result !== notFoundValue) {
          _this2.graph = _this2.graph.set(resourceID, result);
          idsChanged[resourceID] = true;
        }
      });

      this.listeners.forEach(function (callback) {
        callback(idsChanged);
      });
    }

    /**
     * Add actions that will be connected to reducers.
     *
     * @param  {Object} actionSets An object mapping action set identifiers to action functions
     */
  }, {
    key: 'registerActionSets',
    value: function registerActionSets(actionSets) {
      var _this3 = this;

      Object.keys(actionSets).forEach(function (actionSetID) {
        _this3.actionSetIDsToActionFunctions[actionSetID] = actionSets[actionSetID];
      });

      delete this.allConnectedActionSets;
    }

    /**
     * Get a set of actions, connected to this store ready to dispatch to the reducers.
     *
     * @param  {String} actionSetID The unique identifier for the action set
     * @return {Object}             The connected action functions
     */
  }, {
    key: 'getConnectedActionSet',
    value: function getConnectedActionSet(actionSetID) {
      var _this4 = this;

      var sourceActionFunctions = this.actionSetIDsToActionFunctions[actionSetID];
      var connectedActionFunctions = {};

      Object.keys(sourceActionFunctions).forEach(function (actionID) {
        if (actionID === 'introspection') {
          return;
        }

        var sourceActionFunction = sourceActionFunctions[actionID];

        connectedActionFunctions[actionID] = function (payload) {
          // Synchronous, immediately dispatched
          if (sourceActionFunction.length <= 1) {
            var result = sourceActionFunction(payload);
            // Allow empty action declarations
            if (typeof result !== 'undefined') {
              payload = result;
            }
            _this4.dispatch({ actionSetID: actionSetID, actionID: actionID, payload: payload });
          }
          // Asychronous, delegates the dispatching
          else {
              var allActionSets = _this4.getAllConnectedActionSets();
              var currentActionSet = allActionSets[actionSetID];

              var getConsensus = function getConsensus(_ref2) {
                var introspectionID = _ref2.introspectionID;
                var payload = _ref2.payload;
                var combine = _ref2.combine;
                var _ref2$booleanOr = _ref2.booleanOr;
                var booleanOr = _ref2$booleanOr === undefined ? false : _ref2$booleanOr;
                var _ref2$booleanAnd = _ref2.booleanAnd;
                var booleanAnd = _ref2$booleanAnd === undefined ? false : _ref2$booleanAnd;

                if (booleanOr) {
                  combine = function (combined, current) {
                    return combined || current;
                  };
                } else if (booleanAnd) {
                  combine = function (combined, current) {
                    return combined && current;
                  };
                }

                var combinedValue = Object.keys(_this4.resources).reduce(function (combinedValue, resourceID, i) {
                  var resource = _this4.resources[resourceID];
                  var currentValue = (0, _callAction2['default'])({
                    responder: resource.reducer,
                    type: _types.INTROSPECTION_TYPE,
                    initialValue: _this4.graph.get(resourceID),
                    props: resource.props,
                    actionID: introspectionID,
                    actionSetID: actionSetID,
                    payload: payload,
                    notFoundValue: notFoundValue
                  });

                  if (currentValue !== notFoundValue) {
                    if (combinedValue === notFoundValue) {
                      return currentValue;
                    } else {
                      return combine(combinedValue, currentValue);
                    }
                  }

                  return combinedValue;
                }, notFoundValue);

                if (combinedValue !== notFoundValue) {
                  return combinedValue;
                }
              };

              sourceActionFunction(payload, { currentActionSet: currentActionSet, allActionSets: allActionSets, getConsensus: getConsensus });
            }
        };
      });

      return connectedActionFunctions;
    }

    /**
     * Get multiple actions sets, connected to this store ready to dispatch to the reducers.
     *
     * @param  {Array} actionSetIDs The unique identifiers of the action sets.
     * @return {Object}             The action set identifiers to connected action function sets.
     */
  }, {
    key: 'getConnectedActionSets',
    value: function getConnectedActionSets(actionSetIDs) {
      var _this5 = this;

      return actionSetIDs.reduce(function (connectedActionSets, actionSetID) {
        connectedActionSets[actionSetID] = _this5.getConnectedActionSet(actionSetID);
        return connectedActionSets;
      }, {});
    }

    /**
     * Get all known action sets, connected to this store.
     *
     * @return {Object} The action set identifiers to action function sets.
     */
  }, {
    key: 'getAllConnectedActionSets',
    value: function getAllConnectedActionSets() {
      if (this.allConnectedActionSets) {
        return this.allConnectedActionSets;
      } else {
        this.allConnectedActionSets = this.getConnectedActionSets(Object.keys(this.actionSetIDsToActionFunctions));
        return this.allConnectedActionSets;
      }
    }

    /**
     * Convenience method to register and connect action sets in one go.
     *
     * @param  {Object} actionSets An object mapping action set identifiers to action functions
     * @return {Object}            The action set identifiers to connected action function sets.
     */
  }, {
    key: 'registerAndConnectActionSets',
    value: function registerAndConnectActionSets(actionSets) {
      this.registerActionSets(actionSets);
      return this.getConnectedActionSets(Object.keys(actionSets));
    }

    /**
     * Get the state for a particular reducer.
     *
     * @param  {String} id The unique identifier for a particular reducer
     * @return {Any}    The state value of the reducer at the current time
     */
  }, {
    key: 'get',
    value: function get(id) {
      var _this6 = this;

      if (typeof id === 'undefined') {
        return this.graph.get();
      } else if (typeof id === 'string') {
        return this.graph.get(id);
      } else {
        var _ret = (function () {
          var idToState = {};
          Object.keys(id).forEach(function (id) {
            idToState[id] = _this6.graph.get(id);
          });
          return {
            v: idToState
          };
        })();

        if (typeof _ret === 'object') return _ret.v;
      }
    }

    /**
     * Subscribe to reducer updates.
     *
     * @param  {Function} callback A function accepting an array of changed reducer identifiers
     * @return {Function}          A function used to unsubscribe when finished
     */
  }, {
    key: 'subscribe',
    value: function subscribe(callback) {
      var _this7 = this;

      this.listeners.push(callback);

      return function () {
        _this7.listeners.splice(_this7.listeners.indexOf(callback), 1);
      };
    }
  }]);

  return Flambeau;
})();

exports['default'] = Flambeau;
module.exports = exports['default'];