'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _types = require('./types');

/**
 * Attach multiple reducers, with ids as keys and reducer function sets as values.
 *
 * @param  {Object} reducers An object mapping identifiers to reducer function sets
 * @param  {Object} idToProps    (Optional) the unique attributes for each particular reducer
 */

exports['default'] = function (_ref) {
  var reducers = _ref.reducers;
  var _ref$idToProps = _ref.idToProps;
  var idToProps = _ref$idToProps === undefined ? {} : _ref$idToProps;

  var resources = {};
  var states = {};

  Object.keys(reducers).forEach(function (id) {
    var resource = {
      reducer: reducers[id],
      props: idToProps[id]
    };

    resources[id] = resource;
    states[id] = resource.reducer[_types.GET_INITIAL_STATE](resource.props);
  });

  return { resources: resources, states: states };
};

module.exports = exports['default'];